@@ -5,23 +5,23 @@
 //
 
 //
-// Permission is hereby granted, free of charge, to any person obtaining a copy
-// of this software and associated documentation files (the "Software"), to deal
-// in the Software without restriction, including without limitation the rights
-// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-// copies of the Software, and to permit persons to whom the Software is
-// furnished to do so, subject to the following conditions:
+// Permission is hereby granted, free of charge, to any person obtaining a
+// copy of this software and associated documentation files (the "Software"),
+// to deal in the Software without restriction, including without limitation
+// the rights to use, copy, modify, merge, publish, distribute, sublicense,
+// and/or sell copies of the Software, and to permit persons to whom the
+// Software is furnished to do so, subject to the following conditions:
 //
-// The above copyright notice and this permission notice shall be included in all
-// copies or substantial portions of the Software.
+// The above copyright notice and this permission notice shall be included in
+// all copies or substantial portions of the Software.
 //
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 // AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
-// SOFTWARE.
+// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+// DEALINGS IN THE SOFTWARE.
 //
 
 
@@ -38,6 +38,104 @@ using Camel = Beagle.Util.Camel;
 
 namespace IndexMailTool {
 
+	public class PathFinder {
+
+		private PathFinder () { }
+
+		static public String RootDir {
+			get {
+				String homedir = Environment.GetEnvironmentVariable ("HOME");
+				String dir = Path.Combine (homedir, ".beagle-mail-crawler");
+				if (! Directory.Exists (dir)) {
+					Directory.CreateDirectory (dir);
+					// Make sure that ~/.beagle directory is only
+					// readable by the owner.
+					Mono.Posix.Syscall.chmod (dir,
+								  (Mono.Posix.FileMode) 448);
+				}
+				return dir;
+
+			}
+		}
+
+		static public String LogDir {
+			get {
+				string dir = Path.Combine (RootDir, "Log");
+				if (! Directory.Exists (dir))
+					Directory.CreateDirectory (dir);
+				return dir;
+			}
+		}
+
+		static private String AppDir {
+			get {
+				String dir = Path.Combine (RootDir, "App");
+				if (! Directory.Exists (dir))
+					Directory.CreateDirectory (dir);
+				return dir;
+			}
+		}
+
+		// We probably shouldn't expose this.  Use it only for good, not
+		// for evil.
+		static public String AppDataFileName (String appName, String dataName)
+		{
+			// FIXME: should make sure appName & dataName don't
+			// contain evil characters.
+			return Path.Combine (AppDir,
+					     String.Format ("{0}_-_-{1}", appName, dataName));
+		}
+
+		
+		static public bool HaveAppData (String appName, String dataName)
+		{
+			return File.Exists (AppDataFileName (appName, dataName));
+		}
+
+		static public Stream ReadAppData (String appName, String dataName)
+		{
+			return new FileStream (AppDataFileName (appName, dataName),
+					       FileMode.Open,
+					       FileAccess.Read);
+		}
+
+		static public String ReadAppDataLine (String appName, String dataName)
+		{
+			if (! HaveAppData (appName, dataName))
+				return null;
+
+			StreamReader sr = new StreamReader (ReadAppData (appName, dataName));
+			String line = sr.ReadLine ();
+			sr.Close ();
+
+			return line;
+		}
+
+		static public Stream WriteAppData (String appName, String dataName)
+		{
+			return new FileStream (AppDataFileName (appName, dataName),
+					       FileMode.Create,
+					       FileAccess.Write);
+		}
+
+		static public void WriteAppDataLine (String appName, String dataName, String line)
+		{
+			if (line == null) {
+				String fileName = AppDataFileName (appName, dataName); 
+				if (File.Exists (fileName))
+					File.Delete (fileName);
+				return;
+			}
+
+			StreamWriter sw = new StreamWriter (WriteAppData (appName, dataName));
+			sw.WriteLine (line);
+			sw.Close ();
+		}
+	}
+
+
+
+
 	public class LineReader {
 		
 		TextReader reader;
@@ -396,6 +494,7 @@ namespace IndexMailTool {
 	//////////////////////////////////////////////////////////////////	
 
 	public class IndexableMail : Indexable {
+		MailMessage message;
 
 		public IndexableMail (String accountId,
 				      String folderName,
@@ -405,39 +504,46 @@ namespace IndexMailTool {
 			Uri = String.Format ("email://{0}/{1};uid={2}",
 					     accountId, folderName, messageInfo.uid);
 			Type = "MailMessage";
-			MimeType = null;
+			MimeType = "text/plain";
 
+			this.message = message;
 			Timestamp = messageInfo.Date;
 
 			// Assemble the metadata
-			this ["Folder"] = folderName;
-			this ["Subject"] = messageInfo.subject;
-			this ["To"] =  messageInfo.to;
-			this ["From"] = messageInfo.from;
-			this ["Cc"] = messageInfo.cc;
-			this ["Received"] = messageInfo.received.ToString ();
-			this ["SentDate"] = messageInfo.sent.ToString ();
-			this ["Mlist"] = messageInfo.mlist;
+			AddProperty (Property.New ("fixme:folder", folderName));
+			AddProperty (Property.New ("fixme:subject", messageInfo.subject));
+			AddProperty (Property.New ("fixme:to", messageInfo.to));
+			AddProperty (Property.New ("fixme:from", messageInfo.from));
+			AddProperty (Property.New ("fixme:cc", messageInfo.cc));
+			AddProperty (Property.NewDate ("fixme:received", messageInfo.received));
+			AddProperty (Property.NewDate ("fixme:sentdate", messageInfo.sent));
+			AddProperty (Property.New ("fixme:mlist", messageInfo.mlist));
 
 			if (folderName == "Sent")
-				this ["_IsSent"] = "1";
+				AddProperty (Property.NewFlag ("fixme:isSent"));
 
-			this ["_Flags"] = Convert.ToString (messageInfo.flags);
+			AddProperty (Property.NewKeyword ("fixme:flags", messageInfo.flags));
 			
 			if (messageInfo.IsAnswered)
-				this ["_IsAnswered"] = "1";
+				AddProperty (Property.NewFlag ("fixme:isAnswered"));
+
 			if (messageInfo.IsDeleted)
-				this ["_IsDeleted"] = "1";
+				AddProperty (Property.NewFlag ("fixme:isDeleted"));
+
 			if (messageInfo.IsDraft)
-				this ["_IsDraft"] = "1";
+				AddProperty (Property.NewFlag ("fixme:isDraft"));
+
 			if (messageInfo.IsFlagged)
-				this ["_IsFlagged"] = "1";
+				AddProperty (Property.NewFlag ("fixme:isFlagged"));
+			
 			if (messageInfo.IsSeen)
-				this ["_IsSeen"] = "1";
+				AddProperty (Property.NewFlag ("fixme:isSeen"));
+
 			if (messageInfo.HasAttachments)
-				this ["_HasAttachments"] = "1";
+				AddProperty (Property.NewFlag ("fixme:hasAttachments"));
+
 			if (messageInfo.IsAnsweredAll)
-				this ["_IsAnsweredAll"] = "1";
+				AddProperty (Property.NewFlag ("fixme:isAnsweredAll"));
 
 			// Assemble the content, if we have any
 			if (message != null) {
@@ -445,7 +551,7 @@ namespace IndexMailTool {
 				foreach (MailBody body in message.Bodies)
 					BodyToMultiReader (body, multi);
 				if (multi.Count > 0)
-					ContentReader = multi;
+					SetTextReader (multi);
 			}
 		}
 
@@ -472,32 +578,48 @@ namespace IndexMailTool {
 
 	public class MailScanner {
 
-		IndexDriver driver = new IndexDriver ();
+		Indexer indexer = Indexer.Get ();
 		ArrayList toIndex = new ArrayList ();
 		int count = 0;
+		private bool dumbterm = false;
+
+		public MailScanner ()
+		{
+			if (Environment.GetEnvironmentVariable ("TERM") == "dumb")
+				dumbterm = true;
+		}
+
+		private void IndexList (ICollection indexables) 
+		{
+			foreach (Indexable i in toIndex)
+				indexer.Index (i);
+			toIndex = new ArrayList ();
+		}
+			
 
 		private void Schedule (Indexable indexable)
 		{
 			toIndex.Add (indexable);
 			++count;
 			if (toIndex.Count > 1000) {
-				driver.Add (toIndex);
-				driver.Optimize ();
-				toIndex.Clear ();
+				IndexList (toIndex);
 			}
 		}
 
 		public void Flush ()
 		{
 			if (toIndex.Count > 0)
-				driver.Add (toIndex);
+				IndexList (toIndex);
 		}
 
 		//////////////////////////
 
 		public void MessageStatus (String str)
 		{
-			Console.Write ("\x1b[1G{0}\x1b[0K", str); // terminal-fu!
+			if (! dumbterm)
+				Console.Write ("\x1b[1G{0}\x1b[0K", str); // terminal-fu!
+			else
+				Console.WriteLine (str);
 		}
 
 		public void MessageStatus (String format, params object[] args)
@@ -507,7 +629,10 @@ namespace IndexMailTool {
 
 		public void MessageFinished (String str)
 		{
-			Console.WriteLine ("\x1b[1G{0}\x1b[0K", str); // terminal-fu!
+			if (! dumbterm)
+				Console.WriteLine ("\x1b[1G{0}\x1b[0K", str); // terminal-fu!
+			else
+				Console.WriteLine (str);
 		}
 
 		public void MessageFinished (String format, params object[] args)
@@ -569,9 +694,9 @@ namespace IndexMailTool {
 
 					// Parse an RFC 2822 message from the array of lines
 					MailMessage msg = new MailMessage (reader, mi.size);
-
-					//Console.WriteLine ("From: {0}", mi.from);
-					//Console.WriteLine ("Subject: {0}", mi.subject);
+					
+					Console.WriteLine ("From: {0}", mi.from);
+					Console.WriteLine ("Subject: {0}", mi.subject);
 
 					Schedule (new IndexableMail ("local@local",
 								     folderName,
@@ -586,8 +711,8 @@ namespace IndexMailTool {
 			if (mboxStream != null)
 				mboxStream.Close ();
 
-			if (latestTime != lastTime)
-				PathFinder.WriteAppDataLine ("IndexMail", dataName, latestTime.Ticks.ToString ());
+			//			if (latestTime != lastTime)
+				//				PathFinder.WriteAppDataLine ("IndexMail", dataName, latestTime.Ticks.ToString ());
 		}
 
 	}
