@@ -31,9 +31,13 @@ using System.Threading;
 using Gtk;
 
 using Beagle;
+using BU = Beagle.Util;
 
 class QueryTool {
 
+	static int count = 0;
+	static Query query = null;
+
 	static void WriteHit (Hit hit)
 	{
 		Console.WriteLine ("  Uri: {0}", hit.Uri);
@@ -52,29 +56,59 @@ class QueryTool {
 		Console.WriteLine ();
 	}
 
-	static void Main (String[] args) 
+	static void OnGotHits (ICollection hits)
+	{
+		foreach (Hit hit in hits) {
+			WriteHit (hit);
+			++count;
+		}
+	}
+
+	static void QuitWithCount (string type)
+	{
+		Console.WriteLine ("{0} after {1} hit{2}.",
+				   type,
+				   count,
+				   count != 1 ? "s" : "");
+	}
+
+	static void OnFinished ()
+	{
+		Console.WriteLine ("Finished after {0} hits.", count);
+		query.Dispose ();
+		Gtk.Application.Quit ();
+	}
+
+	static void OnCancelled ()
+	{
+		Console.WriteLine ("Cancelled after {0} hits.", count);
+		query.Dispose ();
+		Gtk.Application.Quit ();
+	}
+
+	static void Main (string[] args) 
 	{
 		Gtk.Application.Init ();
 
-		QueryDriver driver = new QueryDriver ();
-		driver.AutoPopulateHack ();
+		query = Beagle.Factory.NewQuery ();
+		query.GotHitsEvent += OnGotHits;
+		query.FinishedEvent += OnFinished;
+		query.CancelledEvent += OnCancelled;
 
-		Query query = new Query (String.Join (" ", args));
-
-		QueryResult result = driver.Query (query);
-		result.Start ();
-
-		Thread.Sleep (1000);
-		result.Wait ();
-
-		foreach (Hit hit in result.Hits) {
-			WriteHit (hit);
+		int i = 0;
+		while (i < args.Length) {
+			if (args [i].StartsWith ("mimetype:")) {
+				string mt = args [i].Substring ("mimetype:".Length);
+				query.AddMimeType (mt);
+			} else {
+				query.AddTextRaw (args [i]);
+			}
+			++i;
 		}
 
-		Console.WriteLine ("Total hits: {0}", result.Count);
+		query.Start ();
 
-		// FIXME: Works around mono dangling thread bug.
-		Environment.Exit (0);
+		Gtk.Application.Run ();
 	}
 
 
